#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const cac = require("cac");
const fs_1 = require("./fs");
const task_1 = require("./task");
const pathLib = require("path");
const defaultCli = cac();
try {
    require('ts-node/register');
}
catch (error) {
    // ignore
}
defaultCli.option('--config, -c <...file>', 'The Foyfile');
defaultCli.parse();
let foyFile = '';
if (defaultCli.options.config && defaultCli.options.config.length) {
    let config = typeof defaultCli.options.config === 'string'
        ? defaultCli.options.config
        : defaultCli.options.config[0];
    foyFile = pathLib.resolve(process.cwd(), config);
}
else {
    foyFile = pathLib.join(process.cwd(), 'Foyfile.js');
    if (!fs_1.fs.existsSync(foyFile)) {
        foyFile = pathLib.join(process.cwd(), 'Foyfile.ts');
    }
}
if (!fs_1.fs.existsSync(foyFile)) {
    throw new TypeError(`Cannot find Foyfile: ${foyFile}`);
}
require(foyFile);
const taskCli = cac();
taskCli
    .option('--config, -c file', 'The Foyfile.js');
let taskManager = task_1.getGlobalTaskManager();
taskManager.getTasks().forEach(t => {
    let strict = taskManager.globalOptions.strict || t.strict;
    let cmd = taskCli
        .command(t.name, t.desc, { allowUnknownOptions: !strict });
    if (t.optionDefs) {
        t.optionDefs.forEach(def => cmd.option(...def));
    }
    cmd.option('-h, --help', 'helps');
    cmd.action((...args) => {
        let options = args.pop();
        if (options.help) {
            cmd.outputHelp();
            return;
        }
        taskManager.run(t.name, { options, args: taskCli.rawArgs.slice(3) });
    });
});
taskCli.on('command:*', () => {
    console.error(`error: Unknown command \`${taskCli.args.join(' ')}\``);
    process.exit(1);
});
taskCli.help();
let taskArgv = process.argv.slice();
{ // Remove config args
    let configIdx = taskArgv.findIndex(t => t === '--config' || t === '-c');
    if (configIdx >= 0) {
        taskArgv.splice(configIdx, 2);
    }
}
taskCli.parse(taskArgv);
//# sourceMappingURL=cli.js.map