"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const execa = require("execa");
exports.execa = execa;
const pathLib = require("path");
exports.exec = execa.shell;
exports.spawn = execa;
class ShellContext {
    constructor() {
        this._cwd = process.cwd();
        this._env = Object.assign({}, process.env);
    }
    cwd() {
        return this._cwd;
    }
    cd(path) {
        this._cwd = pathLib.resolve(this._cwd, path);
    }
    exec(command, options) {
        return exports.exec(command, Object.assign({ cwd: this._cwd, env: this._env, stdio: 'inherit' }, options)).catch(err => {
            let logger = require('./logger').logger;
            logger.error('Exec failed: ', command);
            throw err;
        });
    }
    spawn(file, args, options) {
        return exports.spawn(file, args, Object.assign({ cwd: this._cwd, env: this._env, stdio: 'inherit' }, options)).catch(err => {
            let logger = require('./logger').logger;
            logger.error('Spawn failed: ', file, args.map(a => `"${a.replace(/"/g, '\\"')}"`).join(' '));
            throw err;
        });
    }
    env(key, val) {
        this._env[key] = val;
        return this._env;
    }
}
exports.ShellContext = ShellContext;
function shell(callback) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        return callback(new ShellContext());
    });
}
exports.shell = shell;
//# sourceMappingURL=exec.js.map