"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const _fs = require("fs");
const util = require("util");
const pathLib = require("path");
function copy(src, dist, opts) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        let srcStat = yield exports.fs.stat(src);
        let isFiltered = true;
        let isOverrided = false;
        if (opts) {
            if (opts.filter)
                isFiltered = opts.filter(srcStat);
            if (opts.override)
                isOverrided = opts.override;
        }
        if (!isFiltered)
            return;
        if (srcStat.isDirectory()) {
            yield exports.fs.mkdirp(dist);
            let childs = yield exports.fs.readdir(src);
            yield Promise.all(childs.map(child => copy(pathLib.join(src, child), pathLib.join(dist, child), opts)));
        }
        else if (srcStat.isFile() || srcStat.isSymbolicLink()) {
            try {
                let stat = yield exports.fs.stat(dist);
                if (isOverrided) {
                    if (stat.isFile() || stat.isSymbolicLink()) {
                        yield exports.fs.unlink(dist);
                    }
                    else if (stat.isDirectory()) {
                        yield exports.fs.rmdir(dist);
                    }
                }
                else
                    return;
            }
            catch (error) {
                let lastChar = dist[dist.length - 1];
                if (lastChar === '/' || lastChar === '\\') {
                    yield exports.fs.mkdirp(dist);
                    dist = pathLib.join(dist, pathLib.basename(src));
                }
                else {
                    let dir = pathLib.dirname(dist);
                    yield exports.fs.mkdirp(dir);
                }
            }
            yield exports.fs.copyFile(src, dist);
        }
    });
}
exports.fs = Object.assign({}, _fs, { access: util.promisify(_fs.access), copyFile: util.promisify(_fs.copyFile), open: util.promisify(_fs.open), rename: util.promisify(_fs.rename), truncate: util.promisify(_fs.truncate), rmdir: util.promisify(_fs.rmdir), mkdir: util.promisify(_fs.mkdir), readdir: util.promisify(_fs.readdir), readlink: util.promisify(_fs.readlink), symlink: util.promisify(_fs.symlink), lstat: util.promisify(_fs.lstat), stat: util.promisify(_fs.stat), link: util.promisify(_fs.link), unlink: util.promisify(_fs.unlink), chmod: util.promisify(_fs.chmod), lchmod: util.promisify(_fs.lchmod), lchown: util.promisify(_fs.lchown), chown: util.promisify(_fs.chown), utimes: util.promisify(_fs.utimes), realpath: util.promisify(_fs.realpath), mkdtemp: util.promisify(_fs.mkdtemp), writeFile: util.promisify(_fs.writeFile), appendFile: util.promisify(_fs.appendFile), readFile: util.promisify(_fs.readFile), existsSync: _fs.existsSync, createReadString: _fs.createReadStream, createWriteStream: _fs.createWriteStream, constants: _fs.constants, copy,
    /**
     * Make directory with parents, like `mkdir -p`
     * @param dir
     */
    mkdirp(dir) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (dir === '/')
                return;
            let parent = pathLib.dirname(dir);
            if (parent && parent !== '/') {
                try {
                    yield exports.fs.stat(parent);
                }
                catch (error) {
                    yield exports.fs.mkdirp(parent);
                }
            }
            try {
                yield exports.fs.stat(dir);
            }
            catch (error) {
                return exports.fs.mkdir(dir);
            }
        });
    },
    /**
     * Remove file or directory recursively, like `rm -rf`
     * @param di
     */
    rmrf(path) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (!exports.fs.existsSync(path))
                return;
            let stat = yield exports.fs.stat(path);
            if (stat.isDirectory()) {
                const children = yield exports.fs.readdir(path);
                yield Promise.all(children
                    .map(child => pathLib.join(path, child))
                    .map(exports.fs.rmrf));
            }
            else {
                yield exports.fs.unlink(path);
            }
        });
    },
    outputFile(path, data, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let dir = pathLib.dirname(path);
            yield exports.fs.mkdirp(dir);
            return exports.fs.writeFile(path, data, options);
        });
    },
    outputFileSync(path, data, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let dir = pathLib.dirname(path);
            yield exports.fs.mkdirp(dir);
            return exports.fs.writeFileSync(path, data, options);
        });
    },
    outputJson(path, data, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return exports.fs.outputFile(path, JSON.stringify(data), options);
        });
    },
    outputJsonSync(path, data, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return exports.fs.outputFileSync(path, JSON.stringify(data), options);
        });
    },
    readJson(path, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let data = yield exports.fs.readFile(path, options);
            if (typeof data !== 'string') {
                data = data.toString('utf8');
            }
            return JSON.parse(data);
        });
    },
    readJsonSync(path, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let data = exports.fs.readFileSync(path, options);
            if (typeof data !== 'string') {
                data = data.toString('utf8');
            }
            return JSON.parse(data);
        });
    } });
//# sourceMappingURL=fs.js.map