import { OptionConfig } from 'cac/types/Option';
import { ShellContext } from './exec';
export interface GlobalOptions {
    /**
     * @default true
     */
    loading?: boolean;
    /**
     * @description Whether task options only allow defined options, default false
     * @default false
     */
    strict?: boolean;
    /**
     * @description log level
     * @default 'debug'
     */
    logLevel?: 'debug' | 'info' | 'warn' | 'error';
}
export declare type OptionDef = [string, string, OptionConfig | undefined];
export declare type TaskFn<O> = (ctx: TaskContext<O>) => void | Promise<void>;
export declare type Dependency = Task | string;
export interface Task<O = any> {
    name: string;
    desc?: string;
    dependencies?: Task[];
    fn?: (ctx: TaskContext<O>) => (void | Promise<void>);
    /**
     * Dependences are executed serially by default.
     * If order doesn't matters and you want better performance via parallel, you can mark it as asynchronized.
     */
    async?: boolean;
    /**
     * Whether rerun it when it occured in dependences tree more then once.
     */
    force?: boolean;
    /**
     * Parsed options
     */
    options: O;
    /**
     * Raw arg strings
     */
    args?: string[];
    /**
     * @description Whether task options only allow defined options, default false
     * @default false
     */
    strict?: boolean;
}
export declare class TaskContext<O = any> extends ShellContext {
    task: Task<O>;
    globalOptions: GlobalOptions;
    constructor(task: Task<O>, globalOptions: GlobalOptions);
    readonly options: O;
}
export declare class TaskManager {
    private _tasks;
    globalOptions: GlobalOptions;
    getTasks(): Task<any>[];
    addTask(task: Task): Task<any>;
    run(name: string, { options, args }: {
        options?: any;
        args?: string[];
    }): Promise<void>;
}
export declare function getGlobalTaskManager(): TaskManager;
/**
 * Set global options for all tasks.
 * @param options
 */
export declare function setGlobalOptions(options: GlobalOptions): void;
declare namespace TaskOptions {
    let last: {
        desc: string;
        optionDefs: [string, string, OptionConfig][];
        strict: boolean;
    };
    function empty(): {
        desc: string;
        optionDefs: [string, string, OptionConfig][];
        strict: boolean;
    };
}
export declare function desc(desc: string): void;
export declare function option(rawName: string, description: string, config?: OptionConfig): void;
export declare function strict(): void;
/**
 * Set options for next task.
 * @param options
 */
export declare function setOption(options: typeof TaskOptions.last): void;
export declare function task<O>(name: string, fn: TaskFn<O>): Task<O>;
export declare function task<O>(name: string, dependencies: Dependency[], fn?: TaskFn<O>): Task<O>;
export {};
